package imm.se.vending.ui;

import imm.se.vending.model.Fruit;
import imm.se.vending.model.NotificationType;
import imm.se.vending.model.VendingMachine;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Observable;
import java.util.Observer;

public class VendingMachineTextUI implements Observer {
    VendingMachine machine;

    public VendingMachineTextUI(int bananas, int apples) {
        machine = new VendingMachine(bananas, apples);
        machine.addObserver(this);
    }

    public VendingMachineTextUI(VendingMachine m) {
        machine = m;
        machine.addObserver(this);
    }

    public static void main(String[] args) throws Exception {
        int bananas = 5;
        int apples = 5;
        if (args.length == 2) {
            bananas = Integer.valueOf(args[0]);
            apples = Integer.valueOf(args[1]);
        }
        new VendingMachineTextUI(bananas, apples).mainLoop(System.in, System.out);
    }

    public void mainLoop(InputStream in, PrintStream out) throws IOException {
        BufferedReader rs = new BufferedReader(new InputStreamReader(in));
        String choice = null;
        do {
            showMenu(out);
            choice = rs.readLine();
            if (choice == null) {
                break;
            }
            int number = Integer.valueOf(choice);
            if (number == 0) {
                break;
            }
            processChoice(number, out);

        } while (choice != null);
    }

    private void processChoice(int number, PrintStream out) {
        switch (number) {
            case 1:
                machine.input(1);
                break;
            case 2:
                machine.input(2);
                break;
            case 3:
                machine.input(5);
                break;
            case 4:
                machine.selectFruit(Fruit.BANANA);
                break;
            case 5:
                machine.selectFruit(Fruit.APPLE);
                break;
            case 6:
                machine.cancel();
                break;
            default:
                break;
        }
    }

    private void showMenu(PrintStream out) {
        out.println("   0) Exit");
        out.println("   1) Input 1 DKK");
        out.println("   2) Input 2 DKK");
        out.println("   3) Input 5 DKK");
        out.println("   4) Select banana");
        out.println("   5) Select apple");
        out.println("   6) Cancel");
        out.println("Select a number (0-6): ");
    }

    public void update(Observable o, Object arg) {
        NotificationType type = (NotificationType) arg;
        if (type == NotificationType.DISPENSED_ITEM) {
            if (machine.getDispensedItem() == Fruit.APPLE) {
                System.out.println("Dispensing: Apple");
            }
            if (machine.getDispensedItem() == Fruit.BANANA) {
                System.out.println("Dispensing: Banana");
            }
            return;
        }
        if (type == NotificationType.REST & machine.getRest() != 0) {
            System.out.println("Rest: DKK " + machine.getRest() );
            return;
        }
        if (type == NotificationType.CURRENT_MONEY) {
            System.out.println("Current Money: DKK " + machine.getCurrentMoney());
            return;
        }
    }
}
